'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = function (_ref) {
  var t = _ref.types;

  var valueToAst = function valueToAst(value) {
    if (typeof value === 'string') {
      return t.stringLiteral(value);
    }

    if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
      var properties = Object.keys(value).map(function (key) {
        var id = t.isValidIdentifier(key) ? t.identifier(key) : t.stringLiteral(key);
        return t.objectProperty(id, valueToAst(value[key]));
      });

      return t.objectExpression(properties);
    }
  };

  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        var source = path.get('source');
        var sourceValue = source.node.value;
        var specifiers = path.get('specifiers');

        if (sourceValue.indexOf('polished') !== 0) return;

        var safeSourceValue = sourceValue.replace(/^polished\/src/, 'polished/lib');
        var importedModule = require(safeSourceValue);

        var invalidatedSpecifiers = specifiers.filter(function (specifier) {
          var importedValue = importedModule;

          if (specifier.node.imported) {
            var imported = specifier.get('imported');
            var importedName = imported.node.name;
            var value = importedValue[importedName];

            if (!value) {
              throw imported.buildCodeFrameError('Method does not exist: ' + methodName);
            }

            importedValue = value;
          }

          var local = specifier.get('local');
          var binding = local.scope.getBinding(local.node.name);
          var refs = binding.referencePaths;

          var invalidatedRefs = refs.filter(function (ref) {
            var matchedMethod = importedValue;

            var callExpression = ref.findParent(function (parent) {
              if (parent.isCallExpression()) {
                return true;
              } else if (parent.isMemberExpression()) {
                var property = parent.get('property');
                var _methodName = property.node.name;
                var method = matchedMethod[_methodName];

                if (!method) {
                  throw property.buildCodeFrameError('Method does not exist: ' + _methodName);
                }

                matchedMethod = method;
                return false;
              } else {
                throw parent.buildCodeFrameError("Unexpected node type: " + parent.type);
              }
            });

            var args = callExpression.get('arguments');
            var foundNonLiteral = args.find(function (arg) {
              return !arg.isLiteral();
            });
            if (foundNonLiteral) return true;

            var serializedArgs = args.map(function (arg) {
              return arg.node.value;
            });
            var result = matchedMethod.apply(undefined, _toConsumableArray(serializedArgs));
            var resultAst = valueToAst(result);

            callExpression.replaceWith(resultAst);
            return false;
          });

          if (!invalidatedRefs.length) {
            specifier.remove();
            return false;
          } else {
            return true;
          }
        });

        if (!invalidatedSpecifiers.length) {
          path.remove();
        }
      }
    }
  };
};

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }